/* ------------------env3.h ---------------------*/
#ifndef ENV3_H
#define ENV3_H

/* constantes */

#define NPMAX    1000          /* Max # of points */

/* definitions de types */

typedef struct PStructure {
  int     x;                  /* abcisse */        
  int     y;                  /* ordonnee */
  int     z;                  /* hauteur  */
  int     num;                /* numero du sommet */
  int     m;                  /* marque booleenne */
  struct AStructure   *art;   /* une arete partant du point */
}Point;

typedef struct AStructure {
  struct PStructure *ori,*ext;            /* origine, extremite */
  struct AStructure  *inv, *suiv, *pred ; /* inverse, suivante (sens direct), predecesseur */
  int     m;                              /* marque booleenne */
}Arete;

/******************************** variables globales ***************************************/
                                    
extern int NP;                     /* nombre de points */
extern int NA;                     /* nombre d'aretes dans ART*/
extern int NS;                     /* nombre de sommets dans SOM */
extern int NF;                     /* nombre de faces dans FACES */
extern Point *POINTS[NPMAX];       /* tableau de tous les  (Point *) */
extern Arete *ART[NPMAX];          /* tableau des (Arete *) */
extern Point *SOM[NPMAX];          /* tableau de tous les sommets */
extern Arete *FACES[NPMAX];        /* tableau des faces */

/*********************************** fonctions **********************************************/

/* allocation memoire */
extern Point *Palloc(void);          /* renvoie un pointeur sur Point */
extern Arete *Aalloc(void);          /* renvoie un pointeur sur Arete */
extern int *ialloc(void);            /* renvoie un pointeur sur int */

/* entrees-sorties */
extern void lire_tab_points(void);   /* remplit le tableau POINTS , affecte la var NP */
extern void ecrire_point(Point *P);  /* ecrit le point *P */
extern void ecrire_arete(Arete *A);  /* ecrit l'arete *A */
extern void ecrire_tab_points(Point *P[],int NP);    /* ecrit les NP points, un par ligne */
extern void ecrire_tab_aretes(Arete *A[],int NA);    /* ecrit les NA points, un par ligne */
extern void ecrire_wrml(void);       /* imprime, sur la sortie standard, le fichier wrml */
extern int succ(int i);              /* index du successeur dans ART */
extern int pred(int i);              /* index du predecesseur dans ART */

/* comparaisons */
extern int index_max(void *tab, int imin, int imax, int sizeofelt, int (*cmp)(const void *,const void *)); 
       /*  compare par ordre cmp; renvoie index du max dans tab[imin..imax] */
extern void qsort();
       /* fonction de tri de la bibli standard */
extern int compare_x(const void *p1, const void *p2);
       /* compare les abcisses; les args sont des (Point **); renvoie -1,0,1 */
extern int compare_moinsx(const void *p1, const void *p2);
       /* compare les (-x); les args sont des (Point **); renvoie -1,0,1 */
extern int compare_y(const void *p1, const void *p2);
       /* compare les ordonnees; les args sont des (Point **); renvoie -1,0,1 */


/* geometrie */
extern int det2(Point *p1, Point *p2);       
       /* determinant vectoriel de la projection sur x,y */
extern int detaff2(Point *pp1, Point *p1, Point *pp2, Point *p2);       
       /* det2 de *p1-*pp1,*p2-*pp2 */
extern int det(Point *p1, Point *p2, Point *p3);       
       /* determinant vectoriel */
extern int detaff(Point *pp1,Point *p1,Point *pp2,Point *p2, Point *pp3,Point *p3);       
       /* det de *p1-*pp1,*p2-*pp2, *p3-*pp3 */
extern Arete *arete_init(int imin, int imax1, int imax2);
/* renvoie une nouvelle arete  reliant conv(imin--imax1) avec conv(imax1+1,imax2) */
extern Arete *vainqueur1(Arete *A);
/* renvoie le vainqueur, partant de A->ori->art, en tournant autour de A->ori
   dans le sens indirect */
extern Arete *vainqueur2(Arete *A);
/* renvoie le vainqueur, partant de A->ext->art, en tournant autour de A->ext
   dans le sens direct */
extern int cylindre(int imin, int imax1, int imax2);
/* construit le cylindre de liaison de conv(imin--imax1) a conv(imax1+1,imax2)*/
/* le range dans ART a partir de 0, renvoie son nombre d'aretes */
extern void fusion(int imin, int imax1, int imax2);
/* fusionne les deux polytopes conv(imin--imax1) et conv(imax1+1,imax2) */
/* somme connexe le long du cylindre (imin, imax1, imax2) */
extern void tetraedre(int imin,int imax);
/* suppose que imax-imin +1=4 */
/* effet: POINTS[imin]->art element d'une carte decrivant conv(imin ..imax)=tetraedre */
extern void enveloppe(int imin, int imax);
/* entree: POINTS trie selon les abcisses */
/* effet: POINTS[imin]->art element d'une carte decrivant conv(imin ..imax) */

/* enumerations */
extern void parcours_face(Arete *A);
/* si A est non-marquee: insere A dans FACES, parcourt la face a gauche de A, marque toutes ses aretes*/
extern void parcours_som(Arete *A);
/* si A->ori est non-marque: insere A->ori dans SOM, marque A->ori, parcourt toutes les faces incidentes,lance parcours_som sur tous ses voisins */ 
extern void parcours_art();
/* parcourt SOM, range tous les brins issus d'un sommet dans ART; calcule NA */
extern int euler(); 
/* retourne la caracteristique d'Euler-Poincare */
#endif
